/*
SD card datalogger

The circuit:
*  SD card attached to SPI bus as follows:
** UNO:  MOSI - pin 11, MISO - pin 12, CLK - pin 13, CS - pin 4 (CS pin can be changed)
and pin #10 (SS) must be an output
*/

#include <SPI.h>
#include <SD.h>
#include <Wire.h>
#include "RTClib.h"
#include <SoftwareSerial.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>

// The shield uses the I2C SCL and SDA pins. On classic Arduinos
// this is Analog 4 and 5 so you can't use those for analogRead() anymore
// However, you can connect other I2C sensors to the I2C bus and share
// the I2C bus.
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// These #defines make it easy to set the backlight color
#define ON 0x1
#define OFF 0x2

const int chipSelect = 4;
boolean BufferComplete = false;
File dataFile;
RTC_DS1307  rtc;
SoftwareSerial sprox(8, 9); // rx, tx

#define GREEN_LED  3
#define RED_LED    2
#define _ON  0
#define _OFF 1
#define WIRE_ADDR 4
 
#define _DEBUG_MODE 
 
// Setup Vars
String FileName = "";
char ControlNum=0;
char Fn[]="MClockCN.txt";
int i=0;
int a=0;
volatile int Corredor;

// make a string for assembling the data to log:
String dataString = "";

uint8_t hourvalue, minutevalue, secondvalue;

//Buffer 4 bytes de sprox
volatile char Buffer[5];
//ID del Control
char ID=0;

byte CorredorL;

void setup()
{
  
  // Open serial communications
  Serial.begin(19200);

  // Serial comunication sprox
  sprox.begin(19200);
  
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  
  lcd.print("MatxoCloc 2.0 ;)");
  
  lcd.setBacklight(ON);
  
  // Init RTC
  Wire.begin();

  //Init Timestamps debug date..only first time
  //rtc.adjust(DateTime(__DATE__, __TIME__));
  //rtc.adjust(DateTime(2014, 3, 30, 23, 10, 00));

  //debugging
  Serial.print("Initializing SD card...");

  // make sure that the default chip select pin is set to output, even if you don't use it:
  // SD Card
  pinMode(10, OUTPUT);  //SS
  // Diagnostic LED's
  pinMode(GREEN_LED, OUTPUT);  //led verd
  pinMode(RED_LED, OUTPUT);    //led vermell 

  //DIP-SW selector ( Control # )
  pinMode(7, INPUT);  // SELECTOR ID
  pinMode(6, INPUT);  // SELECTOR ID
  pinMode(5, INPUT);  // SELECTOR ID
  pinMode(4, INPUT);  // SELECTOR ID

  digitalWrite(RED_LED, _ON);
  digitalWrite(GREEN_LED, _OFF);

  // see if the card is present and can be initialized:
  if (!SD.begin(10,11,12,13)) {  //chipSelect 

    Serial.println("Card failed, or not present");

    // don't do anything more:
    while (1) {
      digitalWrite(RED_LED, _ON);
      delay(100);
      digitalWrite(RED_LED, _OFF);
      delay(100);
    }//fi While  
  }

  //Tot perfecte.. seguim
  Serial.println("card initialized.");

  
  //Generate File Name
  //CALCULAR ID TORRE CONTROL
  ControlNum=0;
  if (!digitalRead(7)){
    ControlNum+=8;
  }
  if (!digitalRead(6)){
    ControlNum+=4;
  }
  if (!digitalRead(5)){
    ControlNum+=2;
  }
  if (!digitalRead(4)){
    ControlNum+=1;
  }

  FileName = "MClock"; 
  FileName += String (ControlNum,DEC);
  FileName += ".txt";

  //Convert to cha array...
  for (i=0;i<=FileName.length();i++) {
    Fn[i] = FileName.charAt(i);
  }
  
  // Open up the file we're going to log to!
  dataFile = SD.open(Fn, FILE_WRITE);
  if (! dataFile) {

    Serial.println("error opening datalog.txt");

    // Wait forever since we cant write data
    while (1) {
      digitalWrite(RED_LED, _ON);
      delay(300);
      digitalWrite(RED_LED, _OFF);
      delay(300);
    }
    
  }


  // start up acabat. End setup signaling...

  digitalWrite(RED_LED, _OFF);

  digitalWrite(GREEN_LED, _ON);
  delay(200);
  digitalWrite(GREEN_LED, _OFF);
  delay(200);
  digitalWrite(GREEN_LED, _ON);
  delay(200);
  digitalWrite(GREEN_LED, _OFF);

void loop()
{


  a=0;
  
  //READ SERIAL PORT SPROX 0f 15 e0 224
  while(a<4){

    // send data only when you receive data:
    if (sprox.available()>0){
      Buffer[a]=sprox.read();
      a+=1;
    }
  }
  
  //GET TIME
  DateTime now = rtc.now();

  //CALCULAR ID TORRE CONTROL
  ID=0;
  if (!digitalRead(7)){
    ID+=8;
  }
  if (!digitalRead(6)){
    ID+=4;
  }
  if (!digitalRead(5)){
    ID+=2;
  }
  if (!digitalRead(4)){
    ID+=1;
  }

  //

  //dataString = "CONTROL:";
  dataString = String (ID,DEC);
  dataString += " / " ;   

  //DORSAL
  Corredor= (Buffer[2]);
  Corredor*=256;
  CorredorL = Buffer[3];
  Corredor += CorredorL;
  
  //dataString += String(Buffer[2]*256 + Buffer[3]);
  dataString += String (Corredor);
  dataString += " / " ; 
 
  //TIMESTAMP:
  //dataString += "TIMESTAMP:";
  hourvalue=now.hour();
  minutevalue=now.minute();
  secondvalue= now.second();

  if (hourvalue<10){
    dataString += "0" ; 
  }

  dataString += String(now.hour());
  dataString += ":" ; 

  if (minutevalue<10){
    dataString += "0" ; 
  }
  dataString += String(now.minute());
  dataString += ":" ;

  if (secondvalue<10){
    dataString += "0" ; 
  } 
  dataString += String(now.second());

  //PRINT LINE IN SD
  dataFile.println(dataString);

  //PRINT LINE IN SERIAL PORT (DEBUG)
  Serial.println(dataString);  //actiu en release

  //PRINT LINE DATA TO GSM SHIELD
  dataString += "\0";
  
  char charBuf[dataString.length()+1];
  dataString.toCharArray(charBuf, dataString.length()+1);
  
  Wire.beginTransmission(WIRE_ADDR); // transmit to device
  Wire.write(charBuf);
  Wire.endTransmission();

  // The following line will 'save' the file to the SD card after every
  // line of data - this will use more power and slow down how much data
  // you can read but it's safer! 
  // If you want to speed up the system, remove the call to flush() and it
  // will save the file only every 512 bytes - every time a sector on the 
  // SD card is filled with data.

  dataFile.flush();

  lcd.clear();
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(0, 0);

  //MEnu run : show -->   Run: XXX - XX:XX
  //                           Control pas: XXX 
  
  lcd.print("Runner:");
  lcd.print("XXX");
  lcd.print("-");
  lcd.print(String(hourvalue));
  lcd.print(":");
  lcd.print(String(minutevalue));
   
  lcd.setCursor(0, 1);

  lcd.print("Control Pas: ");
  lcd.print("XXX");
  
  

  digitalWrite(GREEN_LED, _ON);
  delay(500);
  digitalWrite(GREEN_LED, _OFF);
  delay(500);
  digitalWrite(GREEN_LED, _ON);
  delay(500);
  digitalWrite(GREEN_LED, _OFF);         
}