// include the GSM and Wire libraries
#include <GSM.h>
#include <Wire.h>

// PIN number if necessary
#define PINNUMBER ""

#define RED_LED    12
#define _ON  0
#define _OFF 1
#define ARRAY_LEN 25

// APN information obtained from your network provider (Vodafone)
#define GPRS_APN       "airtelwap.es" // replace with your GPRS APN
#define GPRS_LOGIN     "wap@wap"    // replace with your GPRS login
#define GPRS_PASSWORD  "wap125" // replace with your GPRS password

// initialize the library instance
GSMClient client;
GPRS gprs;
GSM gsmAccess; //(true); // include a 'true' parameter for debug enabled

String request, control, dorsal, hora, input_S;
char input[ARRAY_LEN]; //data coming from i2c
boolean msgReady = false;//data received from i2c
boolean notConnected = true;//GSM connection state
unsigned long myTimeout = 10000; //Time limit in milliseconds for the GSM to be initialized

void setup()
{
  //red led set up and turn on
  pinMode(RED_LED, OUTPUT); 
  digitalWrite(RED_LED, _ON);

  // initialize serial communications
  Serial.begin(19200);
  Serial.println("Starting Arduino web client.");

  // join i2c bus with address #4
  Wire.begin(4);
  // register interruption service routine    
  Wire.onReceive(receiveEvent); 

}

void loop()
{
  // Start GSM shield
  // If your SIM has PIN, pass it as a parameter of begin() in quotes
  unsigned long timeConnect = millis();
  while(notConnected)
  {
    while ((millis() - timeConnect) < myTimeout && notConnected){
      if(gsmAccess.begin(PINNUMBER)==GSM_READY){
        Serial.println("GSM_READY");

        if(gprs.attachGPRS(GPRS_APN, GPRS_LOGIN, GPRS_PASSWORD)==GPRS_READY){
          notConnected = false;
          Serial.println("Connected to GSM");
          digitalWrite(RED_LED, _OFF);
        }
      }
      else{
        Serial.println("Not connected to GSM 1");
        digitalWrite(RED_LED, _ON);
        delay(1000);
      }
    }
    if(notConnected){
      Serial.println("GSM.shutdown");
      gsmAccess.shutdown();
    }
  }

  //if there is a message read to be sent in input, send it
  if(msgReady){
    msgReady = false;
    input_S = String(input);

    //establish connection to server
    if (client.connect("www.centreexcursionistatorello.com", 80))
    {
      //sample data input = 15 / 625 / 23:05:13
      control = input_S.substring(0,input_S.indexOf('/')-1);
      input_S = input_S.substring(input_S.indexOf('/')+2,input_S.length());
      dorsal = input_S.substring(0,input_S.indexOf('/')-1);
      hora = input_S.substring(input_S.indexOf('/')+2,input_S.length());

      //sample php request: ?control=5&dorsal=125&hora=10:11:12
      request =  "GET /mclock_ins.php?control=" + control;
      request += "&dorsal=" + dorsal;
      request += "&hora=" + hora;
      request += " HTTP/1.0";
      
      //send the request through client
      client.println(request);
      client.println("HOST: www.centreexcursionistatorello.com");    
      client.println();
      Serial.println(request);
    } 
    else
    {
      // if you didn't get a connection to the server:
      Serial.println("connection failed");
      notConnected = true;
      digitalWrite(RED_LED, _ON);
    }  
    client.stop();
  }
}

// function that executes whenever data is received from master
// this function is registered as an event, see setup()
void receiveEvent(int howMany)
{
  //read all incoming chars and rise the flag
  for (int i = 0; i < howMany; i++){
    input[i] = Wire.read();
  }
  input[howMany] = '\0';

  msgReady = true;
}









