#!/usr/bin/python
# -*- coding: utf-8 -*-

import math
import time
import commands
import Adafruit_CharLCD as LCD
import threading
import httplib
from httplib import HTTPException
import os.path
import random
import thread
import pygame
import serial
import logging
from collections import deque
from config import *

# Initialize serial port
def serial_setup(ser):
	ser.port = '/dev/ttyAMA0'
	ser.baudrate = 19200
	ser.bytesize = serial.EIGHTBITS
	ser.parity = serial.PARITY_NONE
	ser.stopbits = serial.STOPBITS_ONE
	ser.timeout = 0
	ser.xonxoff = False
	ser.rtscts = False
	ser.dsrdtr = False

# HTTP GET request to php for real-time DB update
def send_online(dorsal, hora):
	try:
		# www.centreexcursionistatorello.com/pusher/mclock_ins.php?control=1&dorsal=1&hora=10:11:12
		conn = httplib.HTTPConnection('www.centreexcursionistatorello.com', 80, timeout=2)
		conn.request('GET', '/pusher/mclock_ins.php?control=' + str(CONTROL_ID)
					+'&dorsal=' + str(dorsal) +'&hora='+hora)
		logging.info('GET ' + '/pusher/mclock_ins.php?control=' + str(CONTROL_ID)
					+'&dorsal=' + str(dorsal) +'&hora='+hora)
		r1 = conn.getresponse()
		logging.info(str(r1.status) + ' ' + r1.reason)
		
		if os.path.exists('not_online'):
			send_online_faileds()
		
	except Exception, e:
		logging.info('GET failed: Writing in non_online ' + str(CONTROL_ID) + ' / ' + str(dorsal) + ' / '  + hora)
		with open('not_online', 'a') as f:
			f.write(str(CONTROL_ID) + ' / ' + str(dorsal) + ' / '  + hora + '\n')

# send HTTP GET request for one of the faileds			
def send_online_faileds():
	
	try:
		with open('not_online', 'r') as f:
			line = f.readline()
			fields = line.split('/')
			dorsal = fields[1].strip()
			hora = fields[2].strip()

		if line != "":
			# www.centreexcursionistatorello.com/pusher/mclock_ins.php?control=1&dorsal=1&hora=10:11:12
			conn = httplib.HTTPConnection('www.centreexcursionistatorello.com', 80, timeout=2)
			conn.request('GET', '/pusher/mclock_ins.php?control=' + str(CONTROL_ID)
						+'&dorsal=' + str(dorsal) +'&hora='+hora)
			logging.info(('GET ' + '/pusher/mclock_ins.php?control=' + str(CONTROL_ID)
						+'&dorsal=' + str(dorsal) +'&hora='+hora))
			r1 = conn.getresponse()
			logging.info(str(r1.status) + ' ' + r1.reason)
		
			with open("not_online","r+") as f:
				d = f.readlines()
				f.seek(0)
				for i in d:
					if i != line:
						f.write(i)
				f.truncate()
		
		with open('not_online', 'r') as f:	
			if len(f.readlines()) == 0:
				os.remove('not_online')
				logging.info('not_online file empty and removed')
	
	except Exception, e:
		logging.info('Error in send_online_not_online() '+control+ ' ' + dorsal + ' ' + temps)
		
# test whether there's Internnet connection		
def test_network():
	lcd.clear()
	lcd.message('Esperant\nxarxa')
	
	# Wait for eth0 to come up
	i = 1
	ethDown = True
	while (i<20 and ethDown):
		logging.info(str(i) + ' output from: hostname -I = ' + commands.getoutput('hostname -I'))
		if (commands.getoutput('hostname -I') == ""):
			time.sleep(10)
		else:
			ethDown = False
		i = i + 1

	try:
		lcd.clear()
		lcd.message('Provant\nxarxa')
		conn = httplib.HTTPConnection('www.centreexcursionistatorello.com', 80, timeout=10)
		conn.request('GET', '/pusher/mclock_ins.php')
		r1 = conn.getresponse()
		logging.info(str(r1.status) + ' ' + r1.reason)
		return 0
	except Exception, e:
		logging.info('No network')
		
	try:
		lcd.clear()
		lcd.message('Provant\nxarxa 2')
		conn = httplib.HTTPConnection('www.centreexcursionistatorello.com', 80, timeout=10)
		conn.request('GET', '/pusher/mclock_ins.php')
		r1 = conn.getresponse()
		logging.info(str(r1.status) + ' ' + r1.reason)
		return 0
	except Exception, e:
		return -1	

# update last8: remove first and add last
def update_last8(tmp,new):
	work = tmp.splitlines()
	work = str(work[0]) + ',' + str(work[1])
	work = work.split(',')
	work.pop(0)
	work.append(str(new).zfill(3))
	logging.info(work)
	return work[0]+','+work[1]+','+work[2]+','+work[3]+'\n'+work[4]+','+work[5]+','+work[6]+','+work[7]

# error message for duplicated match
def already_registered(lcd, N):
	lcd.clear()
	lcd.message(' Ja has fitxat!\n  Anims Matxo!')
	time.sleep(1)
	lcd_message_main(lcd, N)

# switch off display backlight
def dim_display():
	lcd.set_color(0,0,0)


# ALL OK message for a reading
def lcd_message_reading_OK(lcd, reading):
	lcd.clear()	
	lcd.message('Anims '+ reading +'\nJa falta menys')
	time.sleep(1)	

# default message string
def lcd_message_main(lcd, N):
	lcd.clear()	
	lcd.message(DEVICE_NAME+'\n' 
				+ time.strftime('%H:%M:%S', time.localtime()) + '   N='+str(N).zfill(3))

# OK end boot message
def lcd_message_boot_OK(lcd):
	lcd.clear()	
	lcd.message('MatxoClock ready\nBons Matxos!!!')

# store and detect button sequence for reboot and shut-down	
def button_sequence(queue, actual):
	logging.info('Button pressed: ' + actual)
	
	queue.append(actual)
	if len(queue) > 6:
		queue.popleft()
		
	if queue == deque(['SELECT', 'LEFT', 'RIGHT', 'LEFT', 'LEFT', 'SELECT']):
		lcd.clear()
		lcd.message('Reinici!!!\nReinici!!!')
		time.sleep(1)
		lcd.clear()
		lcd.set_color(0,0,0)
		commands.getoutput('sudo reboot')
		
	if queue == deque(['SELECT', 'RIGHT', 'LEFT', 'RIGHT', 'RIGHT', 'SELECT']):
		lcd.clear()
		lcd.message('Apagada!!!\nApagada!!!')
		time.sleep(1)
		lcd.clear()
		lcd.set_color(0,0,0)
		commands.getoutput('sudo shutdown -h now')		

# write a reading line in the workfile
def write_logfile(sprox_reading, reading_time):
	with open('workfile', 'a') as f:
		f.write(str(CONTROL_ID) + ' / ' + str(sprox_reading) + ' / '  + reading_time + '\n')

   
############# SETUP
# Create logfile
logging.basicConfig(filename='app.log',level=logging.INFO)

# Initialize the LCD using the pins
lcd = LCD.Adafruit_CharLCDPlate()

# create a queue for storing button sequences
queue = deque()

# Read the workfile backwords, count records (N), save last 8 records (last8)
# File format: 2 / 220 / 07:42:40
N = 0
last8 = '000,000,000,000\n000,000,000,000'
if os.path.isfile('workfile'):
	for line in reversed(open('workfile').readlines()):
		N = N + 1

		fields = line.split('/')
		dorsal = fields[1].strip().zfill(3)

		if N <= 8:
			if N == 1:
				last8 = dorsal
			elif N == 5:
				last8 = last8 + '\n' + dorsal
			else:
				last8 = last8 + ',' + dorsal
if N < 8:
	if N == 1:
		last8 = '000,000,000,000\n000,000,000,' + last8
	elif N == 2:
		last8 = '000,000,000,000\n000,000,' + last8
	elif N == 3:
		last8 = '000,000,000,000\n000,' + last8
	elif N == 4:
		last8 = '000,000,000,000\n' + last8
	elif N == 5:
		last8 = '000,000,000,' + last8
	elif N == 6:
		last8 = '000,000,' + last8
	elif N == 7:
		last8 = '000,' + last8
	 
logging.info(last8)
logging.info('N = ' + str(N))

# Init Serial Port
ser = serial.Serial()
serial_setup(ser)
ser.open()

# Screen light set to on
lcd.set_color(1,0,0)

# Test if there's network available
if test_network() == 0:
	lcd.clear()
	lcd.message('Sincronitzant\nrellotge')
	logging.info(commands.getoutput('sudo service ntp stop'))
	logging.info(commands.getoutput('sudo ntpd -gq'))
	lcd_message_boot_OK(lcd)
	online_mode = True
	logging.info('Network OK')
	time.sleep(2)
else:
	# disable ntp deamon
	logging.info(commands.getoutput('sudo service ntp stop'))
	logging.info(commands.getoutput('sudo date -s "Sun Apr 19 00:00:00 CEST 2015"'))
	online_mode = False
	logging.info('Network NOK')
	
# Get local IP
localIP=commands.getoutput('hostname -I')	
	
# A timer is created so every DIM_TIME seconds the screen will dim
dim_timer = threading.Timer(DIM_TIME, dim_display)
dim_timer.start()
	
# Welcome screen with default info page
lcd_message_main(lcd, N)	

################ RUN
while True:
	if ser.read() != "":
		data2 = ser.read()
		data3 = ser.read()
		data4 = ser.read()
		sprox_reading = str(ord(data3) * 256 + ord(data4))
		sprox_reading_zfill = str(ord(data3) * 256 + ord(data4)).zfill(3)
		reading_time = time.strftime('%H:%M:%S', time.localtime())
		logging.info('Reading ' + sprox_reading + ' ' + reading_time)

		dim_timer.cancel()
		lcd.set_color(1,0,0)
		
		if last8.find(sprox_reading) == -1:
			lcd_message_reading_OK(lcd, sprox_reading_zfill)
			last8 = update_last8(last8,sprox_reading_zfill)

			write_logfile(sprox_reading, reading_time)
			if (online_mode):
				thread.start_new_thread(send_online,(sprox_reading, reading_time))

			N = N + 1
			logging.info('N = ' + str(N))
			lcd_message_main(lcd, N)
		
		else:
			already_registered(lcd, N)


	if (lcd.is_pressed(LCD.SELECT) or lcd.is_pressed(LCD.RIGHT) or lcd.is_pressed(LCD.LEFT)
	or lcd.is_pressed(LCD.UP) or lcd.is_pressed(LCD.DOWN)):
		dim_timer.cancel()
		lcd.set_color(1,0,0)

		if lcd.is_pressed(LCD.UP):
			button_sequence(queue, "UP")
			lcd_message_main(lcd, N)

		if lcd.is_pressed(LCD.DOWN):
			button_sequence(queue, "DOWN")
			lcd_message_main(lcd, N)

		if lcd.is_pressed(LCD.LEFT):
			button_sequence(queue, "LEFT")
			lcd.clear()
			lcd.message('Local IP\n'+ localIP)
			
		if lcd.is_pressed(LCD.RIGHT):
			button_sequence(queue, "RIGHT")
			lcd_message_main(lcd, N)			
			
		if lcd.is_pressed(LCD.SELECT):
			button_sequence(queue, "SELECT")
			lcd.clear()
			lcd.message(last8)			

		dim_timer = threading.Timer(DIM_TIME, dim_display)
		dim_timer.start()

